@@ -337,9 +337,7 @@ as.igraph.nma_data <- function(x, ..., collapse = TRUE) {
       dplyr::arrange(.data$.trt)
   }
 
-  if (!is.null(x$classes)) {
-    v_all$.trtclass <- x$classes
-  }
+  if (!is.null(x$classes)) v_all$.trtclass <- x$classes
 
   g <- igraph::graph_from_data_frame(e_all, directed = FALSE, vertices = v_all)
   return(g)
@@ -695,6 +693,7 @@ has_indirect <- function(network, trt1, trt2) {
 #'   `FALSE`.
 #' @param show_trt_class Colour treatment nodes by class, if `trt_class` is set?
 #'   Default is `FALSE`.
+#' @param level Display network at the `"treatment"` (default) or `"class"` level.
 #' @param nudge Numeric value to nudge the treatment labels away from the nodes
 #'   when `weight_nodes = TRUE`. Default is `0` (no adjustment to label
 #'   position). A small value like `0.1` is usually sufficient.
@@ -752,8 +751,12 @@ has_indirect <- function(network, trt1, trt2) {
 #'   ggplot2::guides(edge_width = "none", size = "none")
 #'
 plot.nma_data <- function(x, ..., layout, circular,
-                          weight_edges = TRUE, weight_nodes = FALSE,
-                          show_trt_class = FALSE, nudge = 0) {
+                          weight_edges = TRUE,
+                          weight_nodes = FALSE,
+                          show_trt_class = FALSE,
+                          level = c("treatment", "class"),
+                          nudge = 0) {
+  level <- rlang::arg_match(level)
   if (missing(layout) && missing(circular)) {
     layout <- "linear"
     circular <- TRUE
@@ -780,9 +783,28 @@ plot.nma_data <- function(x, ..., layout, circular,
     abort(paste("Treatment classes not specified in network.",
                 "Specify `trt_class` in set_*(), or set show_trt_class = FALSE.", sep = "\n"))
 
+  if(level == "class" && is.null(x$classes))
+    abort(paste("Treatment classes not specified in network.",
+                'Specify `trt_class` in set_*(), or use level = "treatment"', sep = "\n"))
+
   if (!rlang::is_double(nudge, n = 1, finite = TRUE))
     abort("`nudge` must be a single numeric value")
 
+  # network at the class level
+  if (level == "class"){
+    if(has_agd_contrast(x)){
+      x$agd_contrast$.trt <- x$agd_contrast$.trtclass
+    }
+    if(has_agd_arm(x)){
+      x$agd_arm$.trt <- x$agd_arm$.trtclass
+    }
+    if(has_ipd(x)){
+      x$ipd$.trt <- x$ipd$.trtclass
+    }
+    x$classes <- forcats::fct_unique(x$classes)
+    x$treatments <- x$classes
+  }
+
   dat_mixed <- has_ipd(x) && (has_agd_arm(x) || has_agd_contrast(x))
   g <- ggraph::ggraph(igraph::as.igraph(x), layout = layout, circular = circular, ...)
 
