@@ -22,16 +22,29 @@
 #'
 NULL
 
-#' Print DIC details
+#' Methods for `nma_dic` objects
+#'
+#' The `print()` method prints details of DIC model fit statistics, computed by
+#' the  [dic()] function. The `as.data.frame()`, `as_tibble()`, and `as.tibble()` methods return the
+#' pointwise contributions to the DIC and $p_D$ in a data frame or tibble. The `as.array()` and `as.matrix()`
+#' methods returns a 3D MCMC array (as class [mcmc_array]) or matrix of posterior draws of the residual deviances.
 #'
-#' Print details of DIC model fit statistics, computed by [dic()] function.
 #'
 #' @param x An object of class [nma_dic]
-#' @param digits An integer passed to [round()]
-#' @param ... Ignored
+#' @param digits Integer number of digits to display
+#' @param ... Additional arguments passed on to other methods
 #'
-#' @return `x` is returned invisibly.
+#' @rdname nma_dic-methods
+#'
+#' @seealso [dic()], [plot.nma_dic()]
+#'
+#' @return A `data.frame` for `as.data.frame()`, a `tbl_df` for `as.tibble()`
+#'   and `as_tibble()`, a `matrix` for `as.matrix()`, and an `mcmc_array` for
+#'   `as.array()`.
+#'
+#'   The `print()` method returns `x` invisibly.
 #' @export
+#'
 print.nma_dic <- function(x, digits = 1, ...) {
   if (!rlang::is_scalar_integerish(digits)) abort("`digits` must be a single integer.")
 
@@ -51,6 +64,45 @@ print.nma_dic <- function(x, digits = 1, ...) {
   invisible(x)
 }
 
+#' @rdname nma_dic-methods
+#' @export
+as.data.frame.nma_dic <- function(x, ...) {
+  return(as.data.frame(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...)))
+}
+
+#' @rdname nma_dic-methods
+#' @method as.tibble nma_dic
+#' @export
+as.tibble.nma_dic <- function(x, ...) {
+  return(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...))
+}
+
+#' @rdname nma_dic-methods
+#' @export
+as_tibble.nma_dic <- function(x, ...) {
+  return(dplyr::bind_rows(purrr::map(x$pointwise, ~dplyr::select(., -"observed", -"fitted")), ...))
+}
+
+#' @rdname nma_dic-methods
+#' @export
+as.array.nma_dic <- function(x, ...) {
+  out <- x$resdev_array
+  class(out) <-  c("mcmc_array", "array")
+  return(out)
+}
+
+#' @rdname nma_dic-methods
+#' @export
+as.matrix.nma_dic <- function(x, ...){
+  # Follow approach in rstan:::as.matrix.stanfit
+  a <- as.array(x)
+  names_a <- dimnames(a)
+  dim_a <- dim(a)
+  dim(a) <- c(dim_a[1] * dim_a[2], dim_a[3])
+  dimnames(a) <- names_a[-2]
+  class(a) <- "matrix"
+  return(a)
+}
 
 #' Plots of model fit diagnostics
 #'
@@ -63,22 +115,34 @@ print.nma_dic <- function(x, digits = 1, ...) {
 #' @param y (Optional) A second `nma_dic` object, to produce "dev-dev" plots for
 #'   model comparison.
 #' @param ... Additional arguments passed on to other methods
+#' @param type With a single `nma_dic` object, whether to show the residual
+#'   deviance contribution for each data point (`"resdev"`, the default), or a
+#'   leverage plot (`"leverage"`).
 #' @param show_uncertainty Logical, show uncertainty with a `ggdist` plot
-#'   stat? Default `TRUE`.
+#'   stat? Default `TRUE`. Ignored for `type = "leverage"`.
 #' @param stat Character string specifying the `ggdist` plot stat to use if
 #'   `show_uncertainty = TRUE`, default `"pointinterval"`. If `y` is provided,
 #'   currently only `"pointinterval"` is supported.
 #' @param orientation Whether the `ggdist` geom is drawn horizontally
 #'   (`"horizontal"`) or vertically (`"vertical"`). Only used for residual
 #'   deviance plots, default `"vertical"`.
+#' @param dic_contours Numeric vector of DIC contribution contours to show on
+#'   leverage plots, default `1:4`.
 #'
 #' @return A `ggplot` object.
 #' @export
 #'
-#' @details When a single `nma_dic` object is given, a plot of the residual
-#'   deviance contribution for each data point is produced. For a good fitting
-#'   model, each data point is expected to have a residual deviance of 1; larger
-#'   values indicate data points that are fit poorly by the model.
+#' @details When a single `nma_dic` object is given, the default plot (`type =
+#'   "resdev"`) shows the residual deviance contribution for each data point.
+#'   For a good fitting model, each data point is expected to have a residual
+#'   deviance equal to its degrees of freedom (typically 1, except for multi-arm
+#'   trials in contrast format or multinomial outcomes); larger values indicate
+#'   data points that are fit poorly by the model. A leverage plot can be
+#'   produced with `type = "leverage"`, plotting the leverages (contributions to
+#'   \eqn{p_D}) against the signed square root residual deviance, both
+#'   standardised by the degrees of freedom. Contours for different DIC
+#'   contribution cutoffs are also shown; points which lie outside the DIC=3
+#'   contour are generally identified as contributing to a model's poor fit.
 #'
 #'   When two `nma_dic` objects are given, a "dev-dev" plot comparing the
 #'   residual deviance contributions under each model is produced. Data points
@@ -119,6 +183,9 @@ print.nma_dic <- function(x, digits = 1, ...) {
 #'   ggplot2::aes(colour = ggplot2::after_stat(ifelse(ymin > 1, "darkorange", "black"))) +
 #'   ggplot2::scale_colour_identity()
 #'
+#' # Leverage plot for RE model
+#' plot(smk_dic_RE, type = "leverage")
+#'
 #' # Check for inconsistency using UME model
 #' }
 #' @template ex_smoking_nma_re_ume_example
@@ -138,9 +205,11 @@ print.nma_dic <- function(x, digits = 1, ...) {
 #' plot(smk_dic_RE, smk_dic_RE_UME, point_alpha = 0.5, interval_alpha = 0.1)
 #' }
 plot.nma_dic <- function(x, y, ...,
+                         type = c("resdev", "leverage"),
                          show_uncertainty = TRUE,
                          stat = "pointinterval",
-                         orientation = c("vertical", "horizontal", "x", "y")) {
+                         orientation = c("vertical", "horizontal", "x", "y"),
+                         dic_contours = 1:4) {
   # Checks
   has_y <- !missing(y)
 
@@ -154,6 +223,16 @@ plot.nma_dic <- function(x, y, ...,
   if (orientation == "x") orientation <- "vertical"
   else if (orientation == "y") orientation <- "horizontal"
 
+  type <- rlang::arg_match(type)
+  if (type == "leverage") {
+    show_uncertainty <- FALSE
+    if (!rlang::is_vector(dic_contours) || !(is.numeric(dic_contours) || all(is.na(dic_contours))))
+      abort("`dic_contours` must be a numeric vector.")
+    if (!is.null(x$pointwise$ipd) && survival::is.Surv(x$pointwise$ipd[["observed"]]) ||
+        !is.null(x$pointwise$agd_arm) && survival::is.Surv(x$pointwise$agd_arm[["observed"]]))
+      abort("Leverage plots are not currently supported for survival outcomes.")
+  }
+
   # Get resdev samples from resdev_array
   resdev_post <- as.matrix.nma_summary(x$resdev_array) %>%
     tibble::as_tibble()
@@ -332,86 +411,180 @@ plot.nma_dic <- function(x, y, ...,
 
     p <- p + theme_multinma()
 
-  } else { # Produce resdev plot
+  } else {
 
-    if (show_uncertainty) {
-      if (!rlang::is_string(stat))
-        abort("`stat` should be a character string specifying the name of a ggdist stat")
+    if (type == "resdev") { # Produce resdev plot
 
-      stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")
+      if (show_uncertainty) {
+        if (!rlang::is_string(stat))
+          abort("`stat` should be a character string specifying the name of a ggdist stat")
 
-      tb_stat <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
-                          error = function(err) {
-                            abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
-                                        err, sep = "\n"))
-                          })
+        stat <- stringr::str_remove(stat, "^(stat_dist_|stat_|geom_)")
 
-    }
+        tb_stat <- tryCatch(getExportedValue("ggdist", paste0("stat_", stat)),
+                            error = function(err) {
+                              abort(paste("`stat` should be a character string specifying the name of a ggdist stat:",
+                                          err, sep = "\n"))
+                            })
 
-    # Is a horizontal geom specified?
-    horizontal <- orientation == "horizontal"
+      }
 
-    if (horizontal) {
-      resdev_post$.label <- forcats::fct_rev(resdev_post$.label)
+      # Is a horizontal geom specified?
+      horizontal <- orientation == "horizontal"
 
-      p <- ggplot2::ggplot(resdev_post,
-                           ggplot2::aes(y = .data$.label,
-                                        x = .data$resdev)) +
-        ggplot2::labs(x = "Residual Deviance", y = "Data Point")
+      if (horizontal) {
+        resdev_post$.label <- forcats::fct_rev(resdev_post$.label)
 
-      if (dplyr::n_distinct(resdev_post$df) > 1) {
-        if (dplyr::n_distinct(resdev_post$Type) > 1) {
-          p <- p + ggplot2::facet_grid(Type~df_label, space = "free", scales = "free_y") +
-            ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
+        p <- ggplot2::ggplot(resdev_post,
+                             ggplot2::aes(y = .data$.label,
+                                          x = .data$resdev)) +
+          ggplot2::labs(x = "Residual Deviance", y = "Data Point")
+
+        if (dplyr::n_distinct(resdev_post$df) > 1) {
+          if (dplyr::n_distinct(resdev_post$Type) > 1) {
+            p <- p + ggplot2::facet_grid(Type~df_label, space = "free", scales = "free_y") +
+              ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
+          } else {
+            p <- p + ggplot2::facet_grid(df_label~., space = "free", scales = "free_y") +
+              ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
+          }
         } else {
-          p <- p + ggplot2::facet_grid(df_label~., space = "free", scales = "free_y") +
-            ggplot2::geom_vline(ggplot2::aes(xintercept = df), colour = "grey60")
+          p <- p + ggplot2::geom_vline(xintercept = 1, colour = "grey60")
+          if (dplyr::n_distinct(resdev_post$Type) > 1) {
+            p <- p + ggplot2::facet_grid(Type~., space = "free", scales = "free_y")
+          }
         }
+
       } else {
-        p <- p + ggplot2::geom_vline(xintercept = 1, colour = "grey60")
-        if (dplyr::n_distinct(resdev_post$Type) > 1) {
-          p <- p + ggplot2::facet_grid(Type~., space = "free", scales = "free_y")
+        p <- ggplot2::ggplot(resdev_post,
+                             ggplot2::aes(x = .data$.label,
+                                          y = .data$resdev)) +
+          ggplot2::labs(y = "Residual Deviance", x = "Data Point")
+
+        if (dplyr::n_distinct(resdev_post$df) > 1) {
+          if (dplyr::n_distinct(resdev_post$Type) > 1) {
+            p <- p + ggplot2::facet_grid(df_label~Type, space = "free", scales = "free_x") +
+              ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
+          } else {
+            p <- p + ggplot2::facet_grid(.~df_label, space = "free", scales = "free_x") +
+              ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
+          }
+        } else {
+          p <- p + ggplot2::geom_hline(yintercept = 1, colour = "grey60")
+          if (dplyr::n_distinct(resdev_post$Type) > 1) {
+            p <- p + ggplot2::facet_grid(.~Type, space = "free", scales = "free_x")
+          }
         }
       }
 
-    } else {
-      p <- ggplot2::ggplot(resdev_post,
-                           ggplot2::aes(x = .data$.label,
-                                        y = .data$resdev)) +
-        ggplot2::labs(y = "Residual Deviance", x = "Data Point")
-
-      if (dplyr::n_distinct(resdev_post$df) > 1) {
-        if (dplyr::n_distinct(resdev_post$Type) > 1) {
-          p <- p + ggplot2::facet_grid(df_label~Type, space = "free", scales = "free_x") +
-            ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
-        } else {
-          p <- p + ggplot2::facet_grid(.~df_label, space = "free", scales = "free_x") +
-            ggplot2::geom_hline(ggplot2::aes(yintercept = df), colour = "grey60")
-        }
+      if (show_uncertainty) {
+        p <- p + do.call(tb_stat,
+                         args = rlang::dots_list(point_interval = ggdist::mean_qi,
+                                                 ...,
+                                                 orientation = orientation,
+                                                 .homonyms = "last"))
       } else {
-        p <- p + ggplot2::geom_hline(yintercept = 1, colour = "grey60")
-        if (dplyr::n_distinct(resdev_post$Type) > 1) {
-          p <- p + ggplot2::facet_grid(.~Type, space = "free", scales = "free_x")
-        }
+        p <- p + ggplot2::geom_point(...)
       }
-    }
 
-    if (show_uncertainty) {
-      p <- p + do.call(tb_stat,
-                       args = rlang::dots_list(point_interval = ggdist::mean_qi,
-                                               ...,
-                                               orientation = orientation,
-                                               .homonyms = "last"))
-    } else {
-      p <- p + ggplot2::geom_point(...)
-    }
+      p <- p + theme_multinma()
 
-    p <- p + theme_multinma()
+      if (!horizontal) {
+        p <- p + ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5))
+      } else {
+        p <- p + ggplot2::theme(axis.text.y = ggplot2::element_text(hjust = 0, vjust = 0.5))
+      }
+
+    } else { # Leverage plot
+
+      # Calculate signed sqrt resdev, add in leverages
+      if (NROW(x$pointwise$ipd)) {
+        resdev_post_ipd <- dplyr::right_join(resdev_post,
+                                        dplyr::transmute(x$pointwise$ipd,
+                                                         parameter = make_data_labels(.data$.study, .data$.trt),
+                                                         .data$dic,
+                                                         .data$leverage,
+                                                         ssrd =
+                                                           if (is.matrix(.data$observed)) {  # Ordered multinomial case
+                                                            sign(rowSums(.data$observed[,-1] - .data$fitted[,-1], na.rm = TRUE)) * sqrt(.data$resdev)
+                                                           } else {
+                                                             sign(.data$observed - .data$fitted) * sqrt(.data$resdev)
+                                                           }
+                                        ),
+                                        by = "parameter")
+      }
+      if (NROW(x$pointwise$agd_arm)) {
+        resdev_post_aa <- dplyr::right_join(resdev_post,
+                                        dplyr::transmute(x$pointwise$agd_arm,
+                                                         parameter = make_data_labels(.data$.study, .data$.trt),
+                                                         .data$dic,
+                                                         .data$leverage,
+                                                         ssrd =
+                                                           if (is.matrix(.data$observed)) {  # Ordered multinomial case
+                                                             sign(rowSums(.data$observed[,-1] - .data$fitted[,-1], na.rm = TRUE)) * sqrt(.data$resdev)
+                                                           } else {
+                                                             sign(.data$observed - .data$fitted) * sqrt(.data$resdev)
+                                                           }
+                                        ),
+                                        by = "parameter")
+      }
+      if (NROW(x$pointwise$agd_contrast)) {
+        resdev_post_ac <- dplyr::right_join(resdev_post,
+                                        dplyr::transmute(x$pointwise$agd_contrast,
+                                                         parameter = .data$.study,
+                                                         .data$dic,
+                                                         .data$leverage,
+                                                         ssrd = purrr::map2_dbl(.data$observed,
+                                                                                .data$fitted,
+                                                                                ~sign(sum(.x - .y))) * sqrt(.data$resdev)
+                                        ),
+                                        by = "parameter")
+      }
+
+      resdev_post <- dplyr::bind_rows(if (NROW(x$pointwise$ipd)) resdev_post_ipd else tibble::tibble(),
+                                      if (NROW(x$pointwise$agd_arm)) resdev_post_aa else tibble::tibble(),
+                                      if (NROW(x$pointwise$agd_contrast)) resdev_post_ac else tibble::tibble())
+
+      # Standardise sqrt residual deviances and leverages by df (e.g. for multi-arm trials)
+      resdev_post$ssrd <- resdev_post$ssrd / sqrt(resdev_post$df)
+      resdev_post$leverage <- resdev_post$leverage / resdev_post$df
+
+
+      p <- ggplot2::ggplot(resdev_post,
+                           ggplot2::aes(y = .data$leverage,
+                                        x = .data$ssrd,
+                                        label = .data$.label))
+
+      if (dplyr::n_distinct(resdev_post$Type) > 1) {
+        p <- p +
+          ggplot2::aes(colour = .data$Type) +
+          ggplot2::scale_colour_viridis_d("")
+      }
+
+      p <- p +
+        ggplot2::labs(x = "Signed Square Root Residual Deviance",
+                      y = "Leverage")
+
+      rmax <- max(sqrt(resdev_post$resdev))
+
+      if (!all(is.na(dic_contours))) {
+        p <- p +
+          purrr::map(dic_contours,
+                     ~ggplot2::geom_function(fun = function(x, c) c - x^2, args = list(c = .),
+                                            colour = "grey60", inherit.aes = FALSE,
+                                            xlim = c(-rmax, rmax)*1.2)) +
+          ggplot2::annotate("label", vjust = 0, label.size = 0,  colour = "grey60", fill = NA,
+                            x = 0, y = dic_contours, label = paste0("DIC = ", dic_contours))
+      }
+
+      p <- p +
+        ggplot2::geom_point(...) +
+        ggplot2::coord_cartesian(
+          ylim = c(min(0, resdev_post$leverage, na.rm = TRUE),
+                   max(dic_contours, resdev_post$leverage, na.rm = TRUE)),
+          xlim = c(-rmax, rmax)) +
+        theme_multinma()
 
-    if (!horizontal) {
-      p <- p + ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90, hjust = 1, vjust = 0.5))
-    } else {
-      p <- p + ggplot2::theme(axis.text.y = ggplot2::element_text(hjust = 0, vjust = 0.5))
     }
   }
 
